;/*
   F_Create.rexx EXE ColorCube
   QUIT
   _________________________________________________________________________

   ColorCube Demo  2001-2004 by Olivier LAVIALE <olaviale@numericable.fr>

   This example illustrate how to manage colors using the FC_DisplayContext
   object.
*/

#include <libraries/feelin.h>

#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/graphics.h>
#include <proto/utility.h>
#include <proto/feelin.h>

struct LocalObjectData
{
   FAreaData                       *AreaData;
   FPalette                        *Palette;
   UBYTE                            Segments;
};

#define  FA_Cube_Segments                       (FCCA_BASE + 0)

struct FeelinBase                  *FeelinBase;
#define GfxBase                     FeelinBase -> Graphics
#define UtilityBase                 FeelinBase -> Utility

#define FF_Draw_ClearCube           FF_Draw_Custom_1

/*** Private ***************************************************************/

///Cube_Create
struct FeelinPalette * Cube_Create(struct FeelinClass *Class,FObject Obj)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);
   ULONG                  *orgb,x,y,r=0,g=0,b=0,a,step,seg=LOD -> Segments;

   if (_render)
   {
      if (LOD -> Palette)
      {
         F_Do(_display,FM_RemPalette,LOD -> Palette);
         LOD -> Palette = NULL;
      }

      if (orgb = F_New(seg * seg * sizeof (ULONG)))
      {
         step = 256 / (seg - 1);

         for (y = 0 ; y < seg ; y++)
         {
            for (x = 0 ; x < seg ; x ++)
            {
               if ((x == (seg - 1)) && (y == 0))
               {
                  r = 0xFF;
               }
               else
               {
                  a = step * (seg - 1 - y);
                  r = x * (a / (seg - 1));
               }

               if ((y ==(seg - 1)) && (x ==0))
               {
                  b = 0xFF;
               }
               else
               {
                  a = step * (seg - 1 - x);
                  b = y * (a / (seg - 1));
               }

               if ((y == (seg - 1)) && (x == (seg - 1)))
               {
                  g = 0xFF;
               }
               else
               {
                  g = y * ((step * x) / (seg - 1));
               }

               orgb[x+(y*seg)] = (r << 16) | (g << 8) | b;
            }
         }

         LOD -> Palette = (FPalette *) F_Do(_display,FM_AddPalette,seg * seg,orgb);

         F_Dispose(orgb);

         return LOD -> Palette;
      }
   }
   return NULL;
}
//+
///Cube_Delete
void Cube_Delete(struct FeelinClass *Class,FObject Obj)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   if (_render)
   {
      F_Do(_display,FM_RemPalette,LOD -> Palette);

      LOD -> Palette = NULL;
   }
}
//+

/*** Methods ***************************************************************/

///Cube_New
F_METHOD(ULONG,Cube_New)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   LOD -> Segments = 5;
   LOD -> AreaData = (FAreaData *) F_Get(Obj,FA_AreaData);

   return F_SUPERDO();
}
//+
///Cube_Set
F_METHOD(void,Cube_Set)
{
   struct LocalObjectData *LOD  = F_LOD(Class,Obj);
   struct TagItem         *Tags = Msg,
                          *item;

   while  (item = NextTagItem(&Tags))
   switch (item -> ti_Tag)
   {
      case FA_Cube_Segments:
      {
         if (item -> ti_Data > 1 && item -> ti_Data < 17)
         {
            F_Draw(Obj,FF_Draw_Update | FF_Draw_ClearCube);
            LOD -> Segments = item -> ti_Data;
            Cube_Create(Class,Obj);
            F_Draw(Obj,FF_Draw_Update);
         }
         else
         {
            F_Log(FV_ERLV_DEV,"Value %ld out of range (2-16)",item -> ti_Data);
         }
      }
      break;
   }
   F_SUPERDO();
}
//+
///Cube_Show
F_METHOD(ULONG,Cube_Show)
{
   if (F_SUPERDO())
   {
      if (Cube_Create(Class,Obj)) return TRUE;
   }
   return FALSE;
}
//+
///Cube_Hide
F_METHOD(ULONG,Cube_Hide)
{
   Cube_Delete(Class,Obj);

   return F_SUPERDO();
}
//+
///Cube_Draw
F_METHODM(void,Cube_Draw,FS_Draw)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);
   ULONG                   x,y,a,b,p=0,
                           w,h,bonusw,bonush,dw,dh,seg=LOD -> Segments;
   struct RastPort        *rp = _rp;
       
   if (FF_Draw_Object & Msg -> Flags)
   {
      F_SUPERDO();
   }

   if (FF_Draw_ClearCube & Msg -> Flags)
   {
      _FPen(FV_Pen_Fill);
      _Boxf(_mx,_my,_mx2,_my2);
      return;
   }

   y = _my;
   w = _mw / seg; bonusw = _mw - (w * seg);
   h = _mh / seg; bonush = _mh - (h * seg);

   dh = h;

   for (a = 0 ; a < seg ; a++)
   {
      dw = w;
       x = _mx;

      if (a == (seg - 1)) dh = h + bonush;

      for (b = 0 ; b < seg ; b++)
      {
         _APen(LOD -> Palette -> Pens[p]);

         if (b == (seg - 1)) dw = w + bonusw;

         _Boxf(x,y,dw + x - 1,dh + y - 1);

         x += w;
         p++;
      }
      y += h;
   }
}
//+

///Main
void main(void)
{
   if (FeelinBase = (APTR) OpenLibrary("feelin.library",FV_VERSION))
   {
      static struct FeelinMethodEntry Handlers[] =
      {
         (FMethod) Cube_New,   NULL, FM_New,
         (FMethod) Cube_Set,   NULL, FM_Set,
         (FMethod) Cube_Show,  NULL, FM_Show,
         (FMethod) Cube_Hide,  NULL, FM_Hide,
         (FMethod) Cube_Draw,  NULL, FM_Draw,
          NULL
      };

      FClass  *class;
      FObject  app,win,cub,sld;

      if (class = F_CreateClass(FA_Class_LODSize,           sizeof (struct LocalObjectData),
                                FA_Class_Super,             FC_Area,
                                FA_Class_MethodsTable,      Handlers,
                                TAG_DONE))
      {
         app = AppObject,
            FA_Application_Title,        "demo_Cube",
            FA_Application_Version,      "$VER: demo_Cube 2.0 (2003/08/04)",
            FA_Application_Copyright,    " 2000-2003 Olivier LAVIALE",
            FA_Application_Author,       "Olivier LAVIALE <HaploLaMain@aol.com>",
            FA_Application_Description,  "Tutorial on color managing",
            FA_Application_Base,         "demo_Cude",

            Child, win = WindowObject,
               FA_Window_Title,     "Feelin : Cube",
               FA_Window_Open,      TRUE,

               Child, VGroup,
                  Child, cub = F_NewObj(class -> Name,
                                  FA_Frame,       "FP_Frame_Gauge",
                                  FA_MinWidth,    50,
                                  FA_MinHeight,   50,
                                  End,
                  Child, HGroup,
                     Child, TextObject, FA_Text, "Segments :", FA_SetMax,TRUE,DontChain, End,
                     Child, sld = SliderA, TRUE,2,16,5,
                        FA_ID,            MAKE_ID('C','U','B','W'),
                        FA_SetMax,        FV_SetMaxH,
                        FA_ContextHelp,   "Adjust the number of color segment per corner.",
                        End,
                  End,
               End,
            End,
         End;

         if (app)
         {
            F_Do(win,FM_Notify,FA_Window_CloseRequest,TRUE,app,FM_Application_Shutdown,0);
            F_Do(sld,FM_Notify,"FA_Numeric_Value",FV_Notify_Always,cub,FM_Set,2,FA_Cube_Segments,FV_Notify_Value);
            
            F_Do(app,FM_Application_Run);
            F_DisposeObj(app);
         }

         F_DeleteClass(class);
      }

      CloseLibrary(FeelinBase);
   }
   else
   {
      Printf("Unable to open feelin.library v%ld\n",FV_VERSION);
   }
}
//+
