#include "Private.h"

struct FS_UpdateObjects                         { struct FeelinClass *Class; };
struct FS_UpdatePreview                         { struct FeelinClass *Class; ULONG Which; };

///Label_
FObject Label_(STRPTR Name)
{
   return F_NewObj(FC_Text,
                   FA_SetMax,          TRUE,
                   FA_ChainToCycle,    FALSE,
                   FA_Text,            Name,
                   TAG_DONE);
}
//+
///Raster_
FObject Raster_(ULONG Raster)
{
   return   AreaObject,
            FA_ChainToCycle,  FALSE,
            FA_InputMode,     FV_InputMode_Immediate,
            FA_Back,          Raster,
            FA_MinWidth,      4,
            FA_MinHeight,     4,
            End;
}
//+

///hook_update_objects
F_HOOKM(void,hook_update_objects,FS_UpdateObjects)
{
   struct FeelinClass     *Class = Msg -> Class;
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   ULONG idVal = F_IDO(FA_Numeric_Value);

   STRPTR spec;

   if (spec = (STRPTR) F_Get(LOD -> preview,F_IDO(FA_PreviewColor_Spec)))
   {
      switch (*spec)
      {
         case 's':
         {
            LONG val;

            stcd_l(spec+2,&val);

            F_Do(LOD -> g_feelin_slider,FM_Set,idVal,val,FA_NoNotify,TRUE,TAG_DONE);
            F_Set(LOD -> g_masterpage,FA_Group_ActivePage,ID_FEELIN - 1);
         }
         break;

         case 'p':
         {
            LONG val;

            stcd_l(spec+2,&val);

            F_Do(LOD -> g_pen_slider,FM_Set,idVal,val, FA_NoNotify,TRUE, TAG_DONE);
            F_Set(LOD -> g_masterpage,FA_Group_ActivePage,ID_PEN - 1);
         }
         break;

         case 'c':
         {
            LONG val;

            stch_l(spec+2,&val);

            F_Do(LOD -> g_col_r_slider,FM_Set,idVal,(0xFF0000 & val) >> 16,FA_NoNotify,TRUE,TAG_DONE);
            F_Do(LOD -> g_col_g_slider,FM_Set,idVal,(0x00FF00 & val) >>  8,FA_NoNotify,TRUE,TAG_DONE);
            F_Do(LOD -> g_col_b_slider,FM_Set,idVal, 0x0000FF & val       ,FA_NoNotify,TRUE,TAG_DONE);

            F_Set(LOD -> g_masterpage,FA_Group_ActivePage,ID_COLOR - 1);
         }
         break;
      }
   }
   else
   {
      F_Set(LOD -> g_masterpage,FA_Group_ActivePage,0);
      F_Do(LOD -> g_feelin_slider,FM_Set,F_IDO(FA_Numeric_Value),0,FA_NoNotify,TRUE,TAG_DONE);
   }
}
//+
///hook_update_preview

/* Ce hook est uniquement appel par les objets qui permettent de  modifier
les  donnes.  Une dfinition temporaire est cre, puis applique  l'objet
preview qui en fait une  copie.  Cette  copie  est  ensuite  utilise  avec
l'attribut FA_AdjustColor_Spec au cas o il y aurait une notification. */

F_HOOKM(void,hook_update_preview,FS_UpdatePreview)
{
   struct FeelinClass     *Class = Msg -> Class;
   struct LocalObjectData *LOD   = F_LOD(Class,Obj);

   STRPTR spec;
   ULONG idVal = F_IDO(FA_Numeric_Value);

   if (spec = F_New(1024))
   {
      switch (Msg -> Which)
      {
         case ID_FEELIN:
         {
            F_RawFormat(spec,"s:%ld",F_Get(LOD -> g_feelin_slider,idVal));
         }
         break;

         case ID_PEN:
         {
            F_RawFormat(spec,"p:%ld",F_Get(LOD -> g_pen_slider,idVal));
         }
         break;

         case ID_COLOR:
         {
            F_RawFormat(spec,"c:%02lx%02lx%02lx",F_Get(LOD -> g_col_r_slider,idVal),F_Get(LOD -> g_col_g_slider,idVal),F_Get(LOD -> g_col_b_slider,idVal));
         }
         break;
      }

      F_Do(LOD -> preview,FM_Set,
                          FA_NoNotify,    TRUE,
                    F_IDO(FA_PreviewColor_Spec), spec,
                          TAG_DONE);

      F_Dispose(spec);

      F_SuperDo(Class,Obj,FM_Set,F_IDA(FA_AdjustColor_Spec),F_Get(LOD -> preview,F_IDO(FA_PreviewColor_Spec)),TAG_DONE);
   }
}
//+

/*** Methods ***************************************************************/

///AC_New
F_METHOD(APTR,AC_New)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   FObject g_clear;
   FObject g_shine;
   FObject g_halfshine;
   FObject g_fill;
   FObject g_halfshadow;
   FObject g_shadow;
   FObject g_halfdark;
   FObject g_dark;
   FObject g_text;
   FObject g_high;

   /* create object */

   if (F_SuperDo(Class,Obj,Method,

      FA_ChainToCycle,  FALSE,
      FA_Horizontal,    TRUE,

      Child, VGroup, FA_Weight,75, GroupFrame, FA_Frame_Title, F_CAT(TITLE),
         Child, LOD -> preview = PreviewColorObject,
            FA_ChainToCycle,  FALSE,
            FA_Draggable,     TRUE,
            FA_Dropable,      TRUE,
            FA_Frame,         FV_Frame_None,
            FA_InputMode,     FV_InputMode_Immediate,
            FA_MinHeight,     30,
            FA_MinWidth,      30,
            End,
      End,

      Child, VGroup,
         Child, LOD -> g_masterpage = Page,
            Child, VGroup, FA_Group_Title, F_CAT(FEELIN),
               Child, HGroup, FA_Group_HSpacing,0,
                  Child, g_shine       = Raster_(FI_Shine),
                  Child, g_halfshine   = Raster_(FI_HalfShine),
                  Child, g_fill        = Raster_(FI_Fill),
                  Child, g_halfshadow  = Raster_(FI_HalfShadow),
                  Child, g_shadow      = Raster_(FI_Shadow),
                  Child, g_halfdark    = Raster_(FI_HalfDark),
                  Child, g_dark        = Raster_(FI_Dark),
                  Child, BarObject, FA_MinWidth,10, End,
                  Child, g_text        = Raster_((ULONG)("s:0")),
                  Child, g_high        = Raster_(FI_Highlight),
               End,

               Child, LOD -> g_feelin_slider = SliderA, TRUE,0,FV_PEN_COUNT - 1,0, FA_SetMax,FV_SetMaxH, End,
            End,

            Child, HGroup, FA_Group_Title, F_CAT(PEN),
               Child, LOD -> g_pen_slider = SliderA, TRUE,-128,127,0, FA_SetMax,FV_SetMaxH, End,
            End,

            Child, ColGroup(2), FA_Group_Title, F_CAT(RGB),
               Child, Label_(F_CAT(Red)),    Child, LOD -> g_col_r_slider = SliderA, TRUE,0,255,0, FA_SetMax,FV_SetMaxH, End,
               Child, Label_(F_CAT(Green)),  Child, LOD -> g_col_g_slider = SliderA, TRUE,0,255,0, FA_SetMax,FV_SetMaxH, End,
               Child, Label_(F_CAT(Blue)),   Child, LOD -> g_col_b_slider = SliderA, TRUE,0,255,0, FA_SetMax,FV_SetMaxH, End,
            End,
         End,

         Child, g_clear = SimpleButton(F_CAT(CLEAR)),
      End,

      TAG_MORE,Msg))
   {
      ULONG idVal = F_IDO(FA_Numeric_Value);

      F_Do(LOD -> preview,FM_Notify,F_IDO(FA_PreviewColor_Spec),FV_Notify_Always,FV_Notify_Application,FM_Application_PushMethod,5,Obj,FM_CallHookEntry,2,hook_update_objects,Class);

      F_Do(LOD -> g_feelin_slider,     FM_Notify,idVal,FV_Notify_Always,Obj,FM_CallHookEntry,3,hook_update_preview,Class,ID_FEELIN);
      F_Do(LOD -> g_pen_slider,        FM_Notify,idVal,FV_Notify_Always,Obj,FM_CallHookEntry,3,hook_update_preview,Class,ID_PEN);
      F_Do(LOD -> g_col_r_slider,      FM_Notify,idVal,FV_Notify_Always,Obj,FM_CallHookEntry,3,hook_update_preview,Class,ID_COLOR);
      F_Do(LOD -> g_col_g_slider,      FM_Notify,idVal,FV_Notify_Always,Obj,FM_CallHookEntry,3,hook_update_preview,Class,ID_COLOR);
      F_Do(LOD -> g_col_b_slider,      FM_Notify,idVal,FV_Notify_Always,Obj,FM_CallHookEntry,3,hook_update_preview,Class,ID_COLOR);

      F_Do(g_shine,        FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_Shine);
      F_Do(g_halfshine,    FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_HalfShine);
      F_Do(g_fill,         FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_Fill);
      F_Do(g_halfshadow,   FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_HalfShadow);
      F_Do(g_shadow,       FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_Shadow);
      F_Do(g_halfdark,     FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_HalfDark);
      F_Do(g_dark,         FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_Dark);
      F_Do(g_text,         FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_Text);
      F_Do(g_high,         FM_Notify,FA_Selected,TRUE,LOD -> g_feelin_slider,FM_Set,2,idVal,FV_Pen_Highlight);

      F_Do(g_clear,FM_Notify,FA_Pressed,FALSE,Obj,FM_Set,2,F_IDA(FA_AdjustColor_Spec),NULL);

      F_Do(Obj,FM_CallHookEntry,hook_update_objects,Class);

      return Obj;
   }
   return NULL;
}
//+
///AC_Set
F_METHOD(void,AC_Set)
{
   struct LocalObjectData *LOD  = F_LOD(Class,Obj);
   struct TagItem         *Tags = Msg,
                          *tag,item;

   while  (tag = F_DynamicNTI(&Tags,&item,Class))
   switch (item.ti_Tag)
   {
      case FA_AdjustColor_Spec:
      {
         F_Set(LOD -> preview,F_IDO(FA_PreviewColor_Spec),item.ti_Data);
      }
      break;
   }
   F_SUPERDO();
}
//+
///AC_Get
F_METHOD(void,AC_Get)
{
   struct LocalObjectData *LOD  = F_LOD(Class,Obj);
   struct TagItem         *Tags = Msg,item;

   BOOL up=FALSE;

   while  (F_DynamicNTI(&Tags,&item,Class))
   switch (item.ti_Tag)
   {
      case FA_AdjustColor_Spec:
      {
         F_STORE(F_Get(LOD -> preview,F_IDO(FA_PreviewColor_Spec)));
      }
      break;

      default: up = TRUE;
   }

   if (up) F_SUPERDO();
}
//+

