#include "Private.h"

/***************************************************************************/
/*** Private ***************************************************************/
/***************************************************************************/

///pp_create_string
STRPTR pp_create_string(STRPTR String)
{
   if (String)
   {
      STRPTR copy = String;
      ULONG len;

      while (*copy && *copy != ',') copy++;

      if (len = copy - String)
      {
         if (copy = F_New(len + 1))
         {
            CopyMem(String,copy,len);

            return copy;
         }
      }
   }
   return NULL;
}
//+

/***************************************************************************/
/*** Methods ***************************************************************/
/***************************************************************************/

///PreviewColor_New
F_METHOD(ULONG,PreviewColor_New)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   LOD -> AreaData = (FAreaData *) F_Get(Obj,FA_AreaData);

   return F_SUPERDO();
}
//+
///PreviewColor_Dispose
F_METHOD(void,PreviewColor_Dispose)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   F_Dispose(LOD -> color_spec); LOD -> color_spec = NULL;

   F_SUPERDO();
}
//+
///PreviewColor_Set
F_METHOD(void,PreviewColor_Set)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);
   struct TagItem         *Tags = Msg,item;

   while  (F_DynamicNTI(&Tags,&item,Class))
   switch (item.ti_Tag)
   {
      case FA_PreviewColor_Spec:
      {
         if (_render && LOD -> color)
         {
            F_Do(_display,FM_DeleteColor,LOD -> color); LOD -> color = NULL;
         }

         F_Dispose(LOD -> color_spec);
         LOD -> color_spec = pp_create_string((STRPTR)(item.ti_Data));

         if (_render && LOD -> color_spec)
         {
            LOD -> color = (FColor *) F_Do(_display,FM_CreateColor,LOD -> color_spec,F_Get(Obj,FA_ColorScheme));
         }

         F_Draw(Obj,FF_Draw_Object);
      }
      break;
   }

   F_SUPERDO();
}
//+
///PreviewColor_Get
F_METHOD(void,PreviewColor_Get)
{
   struct LocalObjectData *LOD  = F_LOD(Class,Obj);
   struct TagItem         *Tags = Msg,item;

   BOOL up=FALSE;

   while  (F_DynamicNTI(&Tags,&item,Class))
   switch (item.ti_Tag)
   {
      case FA_PreviewColor_Spec:
      {
         F_STORE(LOD -> color_spec);
      }
      break;

      default: up = TRUE;
   }

   if (up) F_SUPERDO();
}
//+

///PreviewColor_Setup
F_METHOD(LONG,PreviewColor_Setup)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   if (F_SUPERDO())
   {
      if (LOD -> color_spec)
      {
         LOD -> color = (FColor *) F_Do(_display,FM_CreateColor,LOD -> color_spec,F_Get(Obj,FA_ColorScheme));
      }

      return TRUE;
   }
   return FALSE;
}
//+
///PreviewColor_Cleanup
F_METHOD(void,PreviewColor_Cleanup)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   if (_render)
   {
      if (LOD -> color)
      {
         F_Do(_display,FM_DeleteColor,LOD -> color); LOD -> color = NULL;
      }
   }
   F_SUPERDO();
}
//+
///PreviewColor_AskMinMax
F_METHOD(ULONG,PreviewColor_AskMinMax)
{
   struct LocalObjectData *LOD = F_LOD(Class,Obj);

   _minw += 5;
   _minh += 5;

   return F_SUPERDO();
}
//+
///PreviewColor_Draw
F_METHODM(void,PreviewColor_Draw,FS_Draw)
{
   struct LocalObjectData    *LOD = F_LOD(Class,Obj);
   struct RastPort           *rp  = _rp;

   WORD  x1 = _mx, x2 = x1 + _mw - 1,
         y1 = _my, y2 = y1 + _mh - 1;

   F_SUPERDO();

   if (_mw > 2 && _mh > 2)
   {
      if (LOD -> color)
      {
         _APen(LOD -> color -> Pen);
         _Boxf(x1,y1,x2,y2);
      }
      else
      {
         _FPen(FV_Pen_Shadow);
         _Move(x1,y1);   _Draw(x2-1,y2);
         _Move(x1+1,y2); _Draw(x2,y1);
         _FPen(FV_Pen_HalfShine);
         _Move(x1+1,y1); _Draw(x2,y2);
         _Move(x1,y2);   _Draw(x2-1,y1);
      }
   }
}
//+

///PreviewColor_DnDQuery
F_METHODM(FObject,PreviewColor_DnDQuery,FS_DnDQuery)
{
   if (F_SUPERDO())
   {
      STRPTR spec;

      if (spec = (STRPTR) F_Get(Msg -> Source,F_IDA(FA_PreviewColor_Spec)))
      {
//         F_Log(0,"0Source %s{%08lx}",_classname(Msg -> Source),Msg -> Source);

         return Obj;
      }
      else if (spec = (STRPTR) F_Get(Msg -> Source,F_IDO(FA_Image_Spec)))
      {
         if ((spec[1] == ':') &&
             (spec[0] == 's' || spec[0] == 'p' || spec[0] == 'c'))
         {
//            F_Log(0,"1Source %s{%08lx}",_classname(Msg -> Source),Msg -> Source);

            return Obj;
         }
      }
   }
   return NULL;
}
//+
///PreviewColor_DnDDrop
F_METHODM(void,PreviewColor_DnDDrop,FS_DnDDrop)
{
   ULONG spec;

   spec = F_Get(Msg -> Source,F_IDA(FA_PreviewColor_Spec));
   if (!spec) spec = F_Get(Msg -> Source,F_IDO(FA_Image_Spec));

   if (spec)
   {
      F_Set(Obj,F_IDA(FA_PreviewColor_Spec),spec);
   }
}
//+

